"""context.py - A simple context stack for Gummworld2."""

__version__ = '$Id: context.py 407 2013-08-12 15:11:30Z stabbingfinger@gmail.com $'
__author__ = 'Gummbum, (c) 2011-2016'


__all__ = ['Context', 'push', 'pop', 'top', 'run', 'cstack']


import gsdl2
from gsdl2.locals import (
    KEYDOWN, KEYUP,
    MOUSEMOTION, MOUSEBUTTONUP, MOUSEBUTTONDOWN,
    JOYAXISMOTION, JOYBALLMOTION, JOYHATMOTION, JOYBUTTONUP, JOYBUTTONDOWN,
    # VIDEORESIZE, VIDEOEXPOSE,
    USEREVENT, QUIT,
    # ACTIVEEVENT,
)


cstack = []


def push(c, do_enter=True):
    if cstack:
        if __debug__:
            print("CONTEXT: suspending {0}".format(
                cstack[-1].__class__.__name__))
        cstack[-1].suspend()
    if __debug__:
        print("CONTEXT: pushing {0}".format(c.__class__.__name__))
    cstack.append(c)
    if do_enter:
        if __debug__:
            print("CONTEXT: enter {0}".format(c.__class__.__name__))
        c.enter()


def pop(n=1):
    for j in range(n):
        if cstack:
            if __debug__:
                print("CONTEXT: pop/exit {0}".format(
                    cstack[-1].__class__.__name__))
            c = cstack[-1]
            del cstack[-1]
            c.exit()
        if cstack:
            if __debug__:
                print("CONTEXT: resume {0}".format(
                    cstack[-1].__class__.__name__))
            cstack[-1].resume()


def top():
    return cstack[-1] if cstack else None


def run(app):
    """Push app onto the context stack and start the run loop.

    To exit the run loop gracefully, call context.pop().
    """
    push(app)
    while cstack:
        c = top()
        c.clock.tick()


class Context(object):
    def __init__(self, clock):
        object.__init__(self)
        self.clock = clock

    def update(self, *args):
        """Called once per frame"""
        pass

    def draw(self, *args):
        """Refresh the screen"""
        pass

    def suspend(self):
        """Called when another context is pushed on top of this one."""
        pass

    def resume(self):
        """Called when another context is popped off the top of this one."""
        pass

    def enter(self):
        """Called when this context is pushed onto the stack."""
        pass

    def exit(self):
        """Called when this context is popped off the stack."""
        pass

    push = staticmethod(push)
    pop = staticmethod(pop)
    top = staticmethod(top)

    def get_events(self):
        """Get events and call the handler. Called automatically by run() each
        time the clock indicates an update cycle is ready.
        """
        for e in gsdl2.event.get():
            typ = e.type
            if typ == KEYDOWN:
                self.on_key_down(e)
            elif typ == KEYUP:
                self.on_key_up(e)
            elif typ == MOUSEMOTION:
                self.on_mouse_motion(e)
            elif typ == MOUSEBUTTONUP:
                self.on_mouse_button_up(e)
            elif typ == MOUSEBUTTONDOWN:
                self.on_mouse_button_down(e)
            elif typ == JOYAXISMOTION:
                self.on_joy_axis_motion(e)
            elif typ == JOYBALLMOTION:
                self.on_joy_ball_motion(e)
            elif typ == JOYHATMOTION:
                self.on_joy_hat_motion(e)
            elif typ == JOYBUTTONUP:
                self.on_joy_button_up(e)
            elif typ == JOYBUTTONDOWN:
                self.on_joy_button_down(e)
            # elif typ == VIDEORESIZE:
            #     self.on_video_resize(e)
            # elif typ == VIDEOEXPOSE:
            #     self.on_video_expose()
            elif typ == USEREVENT:
                    self.on_user_event(e)
            elif typ == QUIT:
                    self.on_quit()
            # elif typ == ACTIVEEVENT:
            #     self.on_active_event(e.gain, e.state)

    ## Override an event handler to get specific events.
    # def on_active_event(self, e):
    #     pass

    def on_joy_axis_motion(self, e):
        pass

    def on_joy_ball_motion(self, e):
        pass

    def on_joy_button_down(self, e):
        pass

    def on_joy_button_up(self, e):
        pass

    def on_joy_hat_motion(self, e):
        pass

    def on_key_down(self, e):
        pass

    def on_key_up(self, e):
        pass

    def on_mouse_button_down(self, e):
        pass

    def on_mouse_button_up(self, e):
        pass

    def on_mouse_motion(self, e):
        pass

    def on_quit(self):
        pass

    def on_user_event(self, e):

        pass

    # def on_video_expose(self):
    #     pass
    #
    # def on_video_resize(self, e):
    #     pass
