"""gummworld2.py - startup and runtime settings file

Usage
=====
Settings in this file should always be accessed like so:
    
    import settings
    if settings.whatever: ...
    settings.whatever = value

This insures that runtime changes will be noticed by other modules that import
the settings module. Of course, one can always make local copies of the values
if static behavior is desired.
"""

import os
import sys

import gsdl2


ARG_FULLSCREEN = int('fullscreen' in sys.argv)
ARG_GRAB = 'grab' in sys.argv


# Tunables

master_volume = 1.0

# numpy is used by ptext; some environments have trouble with it; force it on/off here
use_numpy = True
numpy = None
if use_numpy:
    try:
        import numpy
    except ImportError:
        pass

if 'pypy' in sys.executable:
    # high performance settings
    pass
else:
    # low performance settings
    pass

# CPU

time_step = 1.0 / 60.0      # fixed timestep: physics (dt)

# Display - None values are computed at runtime

game_title = 'gsdl2_tol'
max_fps = (60, 120, 180, 240)[2]
resolution = ((800, 800), (720, 720))[0]    # note: gfx do not scale with display size
display_flags = (0, gsdl2.sdlconstants.SDL_WINDOW_FULLSCREEN)[ARG_FULLSCREEN]
screen = None
screen_rect = None
playfield_rect = None
space = None

# Graphics

image_dir = 'data/image'
font_dir = 'data/font'
img_player = os.path.join(image_dir, '2D spaceship sprites with engines.png')
img_laser_blue = os.path.join(image_dir, 'laserblast_blue.png')
img_laser_orange = os.path.join(image_dir, 'laserblast_orange.png')
img_space1 = os.path.join(image_dir, 'SPACE1x150.png')
img_space2 = os.path.join(image_dir, 'SPACE2x150.png')
img_kamikaze = os.path.join(image_dir, 'Kamikaze.png')
img_enemy_explode = os.path.join(image_dir, 'Explode1x200.png')
img_player_explode = os.path.join(image_dir, 'Explode3x200.png')
img_projectiles = os.path.join(image_dir, 'KABOOMx125.png')
img_mine = os.path.join(image_dir, 'mine_purple.png')
img_star = os.path.join(image_dir, 'star.png')

# Sprite render order
(
    layer_background,
    layer_galaxy,
    layer_star,
    layer_planet,
    layer_explosions,
    layer_floating_text,
    layer_projectiles,
    layer_lasers,
    layer_enemies,
    layer_player,
    layer_ui_text,
) = range(11)

# Sound

sound_dir = 'data/sound'
music_dir = 'data/music'
play_music = True
song_finished = 1           # Song endevent
sfx_info = dict(
    new_stage=(os.path.join(sound_dir, 'new_stage.ogg'), 1.0),
    player_laser=(os.path.join(sound_dir, 'player_shoot.ogg'), 0.2),
    enemy_laser=(os.path.join(sound_dir, 'enemy_shoot.ogg'), 0.2),
    player_hit=(os.path.join(sound_dir, 'player_hit.ogg'), 1.0),
    enemy_hit=(os.path.join(sound_dir, 'enemy_hit.ogg'), 1.0),
)


def music_play(i=None):
    global music_counter, music_now_playing
    if i is None:
        i = music_counter % len(music_info)
    music_counter = i + 1
    info = music_info[i]
    music_now_playing = info['name']
    gsdl2.music.load(info['file'])
    gsdl2.music.play()
    gsdl2.music.set_volume(info['volume'] * master_volume)
music_counter = 0
music_info = (
    dict(name='Floating Cities', file=os.path.join(music_dir, 'Floating Cities.ogg'), volume=0.1),
    dict(name='Cyborg Ninja', file=os.path.join(music_dir, 'Cyborg Ninja.ogg'), volume=0.1),
    dict(name='Exit the Premises', file=os.path.join(music_dir, 'Exit the Premises.ogg'), volume=0.1),
    dict(name='Level Up', file=os.path.join(music_dir, 'Level Up.ogg'), volume=0.1),
    dict(name='Phantom from Space', file=os.path.join(music_dir, 'Phantom from Space.ogg'), volume=0.1),
)
music_now_playing = '<hiss>'

# Fonts and UI

intro_duration = 5.0        # duration of Wave N splash; recommend 4.0 or 5.0

font_info = dict(
    ui=(os.path.join(font_dir, 'RAYGUN.ttf'), 50),
    stage=(os.path.join(font_dir, 'Boogaloo.ttf'), 44),
    # floating_text=(os.path.join(font_dir, 'Babelfish.ttf'), 30),
    ui_small=(os.path.join(font_dir, 'Folks-Bold.ttf'), 20),
    ui_tiny=(os.path.join(font_dir, 'Folks-Bold.ttf'), 13),
    floating_text=(os.path.join(font_dir, 'Folks-Heavy.ttf'), 30),
    # floating_text=(os.path.join(font_dir, 'NUWAB___.ttf'), 30),
)

# Fonts
font_stage = None
color_stage = gsdl2.Color('white')
font_floating_text = None
color_floating_text = gsdl2.Color('orange')
font_ui_small = None
color_ui_small = gsdl2.Color('white')
font_ui_tiny = None
color_ui_tiny = gsdl2.Color('white')

# Font cache
font_cache = None                   # cached font textures; initialized by text module
font_cache_expire = 0.5 * 60        # expire unused textures after N seconds; recommend 1 min (1 * 60)
font_cache_memory = 5 * 2 ** 20     # force expiration if memory is exceeded; recommend 5 MB (10 * 2**20)

# Misc

# Game entity types for collision handling (initialized by enemy and player modules at runtime)
FriendlyShipTypes = ()
FriendlyFireTypes = ()
FriendlyTypes = ()
EnemyShipTypes = ()
EnemyEnergyTypes = ()
EnemyOrdinanceTypes = ()
EnemyTypes = ()

# Player settings - you may tweak these, though you might break something
player_ship = None          # player ship; initialized by game module
player_speed = 250          # pixels/sec: suggest 250
player_turn_rate = 360      # degrees/sec: suggest 360
player_input_rate = 0.1     # seconds between mouse clicks to queue a shot (float)
player_fire_rate = 0.2      # seconds between shots (float)
mouse_queue_depth = 1       # Number of mouse clicks to queue (int=1 or higher; recommend 1 to 2)
laser_shot_speed = 350      # Laser shot move: pixels per second (int or float); suggest faster than player
player_max_shots = 3        # Max shots on screen (int)
initial_ships = 3           # Begin each new game with this many lives (int)
player_spawn = 0, 0         # Set when game instantiates
player_penalty = -1000      # score penalty when ship is lost

# Enemy settings - you may tweak these, though you might break something
""" Some helpful diagrams (hopefully)
-------Formation------- --------Convoy--------- --------Patrol---------
                       |     E                 |                       |
    A A A A A A A A    |        E              |    A A   A A A A A    |
    B B B B B B B B    |           E           |    B B B B B B   B    |
    C C C C C C C C    |        E     E        |                       |
    D D D D D D D D    |          E E E        |              A        |
                       |                       |        B              |
           P           |            p          |           P           |
----------------------- ----------------------- -----------------------
Key:    A = Enemy standby formation, rank 0
        B = Enemy standby formation, rank 1
        C = Enemy standby formation, rank 2
        D = Enemy standby formation, rank 3
        E = Enemy in convoy
        P = Player
"""
enemy_fleet = None          # command center; initialized by game module
standby_y = [               # the screen y position of the ranks in the standby formation
    100, 150, 200, 250,
]
enemy_max_shots = 5         # Max shots on screen per enemy (int)
enemy_fire_rate = 50        # Longest potential delay between laser shots (int); recommend 35-70
enemy_speed = 250.0         # Enemy move: pixels per second (int or float)
enemy_spawn_rate = 0.2      # 1 ship every n seconds (int or float)
enemy_step_formation = 64   # pixels per second to step enemy formation
max_on_patrol = None        # max enemy permitted to be on patrol; initialized by enemyfleet
n_on_patrol = None          # current enemy on patrol; initialized by enemyfleet
n_on_standby = None         # current enemy on standby; initialized by enemyfleet
standby_pos = [             # the x,y position of enemy sprites in standby formation; initialized by enemyfleet
    [],
    [],
    [],
    [],
]
sky = None                  # background, celestial bodies manager

# Game runtime
stage = 0                   # index of enemyfleet._FLEET to start with
score = 0                   # points for performance

# Debug

profile = False                 # run cProfile if True; cProfile does not work with pypy
print_game_states = False       # print messages for game state machine
print_enemy_states = False      # print messages for enemy state machine
print_fleet_states = False      # print messages for fleet state machine
internals_hud = False           # display extra hud items showing game internals
render_rect_overlay = False     # render sprite rects (geometry for first pass collision detection)
render_circle_overlay = False   # render sprite circles (geometry for second pass collisions detection)

# Custom

# os.environ['SDL_VIDEO_CENTERED'] = '1'
bypass_menu = None

# Gummbum's development preferences... remove or add a case for your own.
# You must also touch or remove the identity file in the game root directory to effect this.

iamgumm = os.access('iamgumm', os.F_OK)
if iamgumm:
    print('## CUSTOM SETTINGS FOR Gumm (see settings.py)')
    # See main.py for the effect of bypass_menu:
    bypass_menu = (None, 'game', 'credits')[1]
    intro_duration = 2.0
    stage = 4
    internals_hud = False
    # experimental fullscreen...
    # ARG_FULLSCREEN = 1
    # display_flags = gsdl2.sdlconstants.SDL_WINDOW_FULLSCREEN
    # profile = True

del iamgumm, os
