import gsdl2
from gsdl2.sdlconstants import (
    TTF_STYLE_NORMAL as NORMAL,
    TTF_STYLE_BOLD as BOLD,
    TTF_STYLE_ITALIC as ITALIC,
    TTF_STYLE_UNDERLINE as UNDERLINE,
    TTF_STYLE_STRIKETHROUGH as STRIKETHROUGH,
)


import imagecache
import settings
import sprite


settings.font_cache = imagecache.ImageCache(settings.font_cache_expire, settings.font_cache_memory)
settings.font_cache.enable_age_cap(True)

texts = set()


def update(dt):
    for text in list(texts):
        text.update(dt)
        if not text.alive:
            text.remove()
    settings.font_cache.update()


def clear():
    for text in list(texts):
        text.remove()


def make_floating_score(score_text, pos):
    ts = TextSprite(score_text, settings.font_floating_text, settings.color_floating_text, pos,
                    outline=True, expire=1.0)
    starty_per_sec = -100 * settings.time_step
    endy_per_sec = -10 * settings.time_step
    ts.scale_effect(end_scale=0.5).drifty_effect(start_y=starty_per_sec, end_y=endy_per_sec)


class TextSprite(sprite.TextureSprite):
    layer = 0
    __styles = []

    def __init__(self, text, font, color, pos, expire=None,
                 angle=0.0, scale=1.0, alpha=255, radius=-1, flip=False, anchor='center',
                 style=NORMAL, outline=False, background=None, layer=None, renderer=None,
                 encoding='utf-8', wrap_length=0):
        """create a text sprite

        :param text: str; string to rnder
        :param font: Font; font object
        :param color: Color; color object
        :param pos: sequence, length 2, int or float; (x,y) screen position
        :param expire: int or float, duration sprite is to be active
        :param angle: float; image rotation
        :param scale: float; image scale; smaller < 1.0; larger > 1.0
        :param alpha: int; alpha transparency; 0 - 255
        :param radius: int or float; advanced collision detection
        :param flip: bool; flip image on the vertical axis
        :param anchor: str, rect coord attribute ('center', 'topleft', ...); origin for image rotation
        :param style: int; 0 or styles OR'ed together: BOLD | ITALIC | UNDERLINE | STRIKETHROUGH
        :param outline: bool; True=underline, False=no underline
        :param background: Color; color object
        :param layer: int; sprite layer for ordered rendering
        :param renderer: Renderer; if None the default renderer will be used
        :param encoding: str; character encoding of text
        :param wrap_length: int; max length of a line for text wrapping
        """
        self.alive = True
        self.text = text

        if layer is not None:
            self.layer = layer
        if renderer is None:
            renderer = gsdl2.display.get_renderer()

        self._cache_ident = text
        self._cache_transforms = (
            font, color, style, outline, background, renderer, encoding, wrap_length)
        texture = settings.font_cache.get(self._cache_ident, self._cache_transforms)
        if texture is None:
            save_style = font.get_style()
            save_outline = font.get_outline()
            font.set_outline(outline)
            font.set_style(style)
            surf = font.render(text, color, background=background, encoding=encoding, wrap_length=wrap_length)
            font.set_style(save_style)
            font.set_outline(save_outline)
            texture = gsdl2.texture.Texture(renderer, surf)
            settings.font_cache.add(self._cache_ident, self._cache_transforms, texture, expire=-1)
        sprite.TextureSprite.__init__(
            self, texture, pos, angle=angle, scale=scale, alpha=alpha, radius=radius, flip=flip, anchor=anchor)

        if expire is not None:
            self.misc_effect('_expire', 1, 1, expire, cb_end=self.__expired)

        texts.add(self)
        if settings.space:
            settings.space.add(self)

    def update(self, dt):
        if self.alive:
            self.update_pos(self.pos + (self.driftx, self.drifty))

    def __expired(self, *args):
        self._cb_end(*args)
        self.remove()

    def remove(self):
        self.alive = False
        if self in texts:
            texts.remove(self)
        if settings.space:
            settings.space.remove(self)
        settings.font_cache.add(self._cache_ident, self._cache_transforms, self.texture, expire=-1)
